@@ -30,7 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *   collect all the headers.
  *
  *****
- * $Id: analyser.c,v 1.77 2009/03/16 21:41:23 gumpu Exp $
+ * $Id: analyser.c,v 1.78 2009/03/23 21:15:04 gumpu Exp $
  */
 
 #include <stdlib.h>
@@ -1304,29 +1304,38 @@ static void Copy_Lines_To_Item(
 }
 
 /*RZ*/
-#ifdef EXPERIMENTAL
 static void Add_Lines_To_Item(
-        struct RB_Item *arg_item,
-        char *c )
+    struct RB_Item *arg_item,
+    char *c )
 {
-    struct RB_Item_Line *itemline = NULL;
+  struct RB_Item_Line *itemline = NULL;
 
-    arg_item->no_lines++;
-    arg_item->lines = realloc( arg_item->lines, arg_item->no_lines * sizeof( struct RB_Item_Line * ) );
+  if (arg_item->no_lines == 0) {
+    arg_item->lines = malloc( sizeof( struct RB_Item_Line * ) );
     itemline = malloc( sizeof( struct RB_Item_Line ) );
     if ( !itemline || !arg_item->lines) {
-        RB_Panic( "Out of memory! %s (2)\n", "Add_Lines_To_Item" );
+      RB_Panic( "Out of memory! %s (2)\n", "Add_Lines_To_Item" );
     }
-    itemline->line = RB_StrDup( c );
-    itemline->kind = ITEM_LINE_PLAIN;
+    itemline->line = RB_StrDup( "" );
+    itemline->kind = ITEM_LINE_END;
     itemline->format = 0;
-    if ( arg_item->no_lines > 1 ) {
-        arg_item->lines[arg_item->no_lines-1] = arg_item->lines[arg_item->no_lines-2];
-    }
-    arg_item->lines[arg_item->no_lines-2] = itemline;
+    arg_item->lines[0] = itemline;
+    arg_item->no_lines = 1;
+  }
+  arg_item->no_lines++;
+  arg_item->lines = realloc( arg_item->lines, arg_item->no_lines * sizeof( struct RB_Item_Line * ) );
+  itemline = malloc( sizeof( struct RB_Item_Line ) );
+  if ( !itemline || !arg_item->lines) {
+    RB_Panic( "Out of memory! %s (2)\n", "Add_Lines_To_Item" );
+  }
+  itemline->line = RB_StrDup( c );
+  itemline->kind = ITEM_LINE_PLAIN;
+  itemline->format = 0;
+  if ( arg_item->no_lines > 1 ) {
+    arg_item->lines[arg_item->no_lines-1] = arg_item->lines[arg_item->no_lines-2];
+  }
+  arg_item->lines[arg_item->no_lines-2] = itemline;
 }
-#endif
-
 
 /****f* Analyser/RB_Analyse_Items
  * FUNCTION
@@ -1344,12 +1353,14 @@ static int Analyse_Items(
     enum ItemType       item_type = NO_ITEM;
     struct RB_Item     *new_item;
     struct RB_Item     *cur_item;
-#ifdef EXPERIMENTAL
-/*RZ*/
+/*RZ variables for source comment adding */
 int isSource=0;
-struct RB_Item *func_item = NULL;
-char *c;
-#endif
+struct RB_Item *use_item = NULL;
+unsigned int marker;
+char *cur_mchar = NULL;
+char *cur_char = NULL;
+int length;
+
     RB_Item_Lock_Reset(  );
 
     /* find the first item */
@@ -1367,14 +1378,14 @@ char *c;
     {
         new_item = RB_Create_Item( item_type );
         new_item->begin_index = line_nr;
-#ifdef EXPERIMENTAL
-/*RZ*/
-if (strcmp(configuration.items.names[item_type],"FUNCTION")==0) {
-  func_item =  new_item;
+/*RZ add source comments to this item */
+if (use_source_comments) {
+  if (strcmp(configuration.items.names[item_type],use_source_comments)==0) {
+    use_item = new_item;
+  } else {
+    isSource = !strcmp(configuration.items.names[item_type],"SOURCE");
+  }
 }
-isSource= !strcmp(configuration.items.names[item_type],"SOURCE");
-#endif
-
         /* Add the item to the end of the list of items. */
         if ( arg_header->items )
         {
@@ -1392,19 +1403,36 @@ isSource= !strcmp(configuration.items.names[item_type],"SOURCE");
         /* Find the next item */
         for ( ++line_nr; line_nr < arg_header->no_lines; ++line_nr )
         {
-#ifdef EXPERIMENTAL
-/*RZ*/
-if (isSource && func_item) {
-  c = RB_Skip_Whitespace (arg_header->lines[line_nr].line);
-  if (strlen(c) > 2) {
-    if (c[0]=='#') {/* shell comments */
-      Add_Lines_To_Item( func_item, &c[1]);
-    } else if (c[0]=='/' && c[1]=='/') {/* C-comments */
-      Add_Lines_To_Item( func_item, &c[2]);
+/*RZ add comment line to item */
+if (isSource) {
+  cur_char = RB_Skip_Whitespace (arg_header->lines[line_nr].line);
+  for (marker = 0; marker < configuration.source_line_comments.number;
+	 marker++) {
+    cur_mchar = configuration.source_line_comments.names[marker];
+    length = strlen(cur_mchar);
+    if (strncmp(cur_char,cur_mchar,length) == 0) {
+      /* create item if not already existing */
+      if (use_item == NULL) {
+        item_type = RB_Get_Item_Type (use_source_comments);
+        if ( item_type != NO_ITEM ) {
+          use_item = RB_Create_Item( item_type );
+          use_item->no_lines = 0;
+          /* Add the item to the end of the list of items. */
+          if ( arg_header->items ) {
+              for ( cur_item = arg_header->items; cur_item->next;
+                    cur_item = cur_item->next ) {
+                  /* Empty */
+              }
+              cur_item->next = use_item;
+          } else {
+              arg_header->items = use_item;
+          }
+        }
+      }
+      Add_Lines_To_Item( use_item, &cur_char[length]);
     }
   }
 }
-#endif
             item_type = RB_Is_ItemName( arg_header->lines[line_nr].line );
             if ( item_type != NO_ITEM )
             {
@@ -1424,7 +1452,10 @@ if (isSource && func_item) {
          *   Dump_Item( new_item );
          */
     }
-
+/*RZ*/
+if (use_item) {
+  Analyse_Item_Format( use_item );
+}
     return 0;
 }
 
