@@ -1,4 +1,31 @@
-// vi: spell ff=unix
+/* vi: spell ff=unix
+ * */
+
+
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
+
+
 /****h* ROBODoc/Generator
  * FUNCTION
  *   This contains routines to generate the documentation from the
@@ -49,7 +76,7 @@
  *   were the function pointers are initialized at program start based
  *   on the output mode.
  *******
- * $Id: generator.c,v 1.105 2007/05/10 09:45:27 thuffir Exp $
+ * $Id: generator.c,v 1.114 2008/07/29 07:50:56 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -88,7 +115,6 @@
 
 
 /* Module functions */
-// static void         RB_Generate_Empty_Item( FILE * dest_doc );
 
 static void         Generate_Item_Line(
     FILE *dest_doc,
@@ -104,7 +130,6 @@ static void         Generate_Index(
 
 static void         Generate_Header(
     FILE *f,
-    struct RB_Document *document,
     struct RB_header *header,
     char *docname );
 
@@ -893,7 +918,7 @@ void Generate_Item_Name(
         RB_ASCII_Generate_Item_Name( dest_doc, name );
         break;
     case TROFF:
-        RB_TROFF_Generate_Item_Name( dest_doc, name, item_type );
+        RB_TROFF_Generate_Item_Name( dest_doc, name, Works_Like_SourceItem(item_type) );
         break;
         break;
     case UNKNOWN:
@@ -1121,7 +1146,7 @@ void RB_Name_Headers(
 /****f* Generator/RB_Sort_Items
  * FUNCTION
  *   Sort the items in all the headers according to the order
- *   specified in the 'item order' block in the robodoc.rc 
+ *   specified in the 'item order' block in the robodoc.rc
  *   file.
  * SYNOPSIS
  */
@@ -1294,8 +1319,6 @@ void RB_Generate_Documentation(
 {
     RB_SetCurrentFile( NULL );
 
-
-
     if ( document->actions.do_singledoc )
     {
         RB_Generate_SingleDoc( document );
@@ -1346,6 +1369,7 @@ void RB_Generate_MultiDoc(
         RB_Document_Sort_Headers( document );
     }
     RB_Document_Link_Headers( document );
+    RB_Loop_Check( document );
     RB_Fill_Header_Filename( document );
     RB_Name_Headers( document->headers, document->no_headers );
     RB_Sort_Items( document->headers, document->no_headers );
@@ -1450,6 +1474,7 @@ void RB_Generate_SingleDoc(
         RB_Document_Sort_Headers( document );
     }
     RB_Document_Link_Headers( document );
+    RB_Loop_Check( document );
     RB_Fill_Header_Filename( document );
     RB_Name_Headers( document->headers, document->no_headers );
     RB_Sort_Items( document->headers, document->no_headers );
@@ -1582,10 +1607,9 @@ void RB_Generate_Section(
  */
 {
     unsigned long       i;
-    struct RB_header   *header;
     char               *headername;
 
-    // We pass either modulename/name or just the name
+    /* We pass either modulename/name or just the name */
     if ( course_of_action.do_sectionnameonly )
     {
         headername = parent->function_name;
@@ -1631,10 +1655,10 @@ void RB_Generate_Section(
 
     RB_Generate_Nav_Bar( document, document_file, parent );
     RB_Generate_Index_Entry( document_file, document->doctype, parent );
-    Generate_Header( document_file, document, parent, "dummy" );
+    Generate_Header( document_file, parent, document->singledoc_name );
     for ( i = 0; i < document->no_headers; ++i )
     {
-        header = ( document->headers )[i];
+        struct RB_header *header = ( document->headers )[i];
         if ( header->parent == parent )
         {
             RB_Generate_Section( document_file, header, document, depth + 1 );
@@ -1705,7 +1729,7 @@ void RB_Generate_Part(
         document_file = RB_Generate_Header_Start( document_file, i_header );
         RB_Generate_Nav_Bar( document, document_file, i_header );
         RB_Generate_Index_Entry( document_file, document->doctype, i_header );
-        Generate_Header( document_file, document, i_header, docname );
+        Generate_Header( document_file, i_header, docname );
         RB_Generate_Header_End( document_file, i_header );
     }
 }
@@ -1759,22 +1783,7 @@ void RB_Generate_DOT_Image_Link(
         break;
 
     case LATEX:
-        {
-            char                str[TEMP_BUF_SIZE];
-
-            // First we need to convert graph to PDF
-            // (maybe the user is using pdflatex)
-#if defined(RB_MSVC)
-            sprintf( str, "%s %s%d.%s", EPSTOPDF_NAME,
-                     DOT_GRAPH_NAME, dot_nr, dot_type );
-#else
-            snprintf( str, sizeof( str ), "%s %s%d.%s", EPSTOPDF_NAME,
-                      DOT_GRAPH_NAME, dot_nr, dot_type );
-#endif
-            system( str );
-
-            fprintf( f, "\\includegraphics{%s%d}\n", DOT_GRAPH_NAME, dot_nr );
-        }
+        fprintf( f, "\\includegraphics{%s%d}\n", DOT_GRAPH_NAME, dot_nr );
         break;
 
     default:
@@ -1784,25 +1793,95 @@ void RB_Generate_DOT_Image_Link(
 
 /******/
 
+/****f* Generator/RB_Generate_Item_Line_Number
+ * FUNCTION
+ *   Generate line numbers for SOURCE like items
+ * SYNOPSIS
+ */
+void RB_Generate_Item_Line_Number(
+    FILE *dest_doc,
+    int arg_line_number,
+    int max_lines )
+/*
+ * INPUTS
+ *   o dest_doc    -- the file to write to.
+ *   o line_number -- the actual line number.
+ *   o max_lines   -- the maximal line number in this item.
+ * SOURCE
+ */
+{
+    /* First check if we should generate line numbers at all */
+    if ( course_of_action.do_source_line_numbers )
+    {
+        char                fmt[TEMP_BUF_SIZE], str[TEMP_BUF_SIZE];
+        int                 len;
+
+        /* produce the maximal line number as a string */
+        snprintf( str, sizeof( str ), "%d", max_lines );
+        /* get the length of the maximal line number */
+        len = strlen( str );
+        /* create the formatting string for the line numbers */
+        snprintf( fmt, sizeof( fmt ), "%%%dd ", len );
+        /* and create the line number as a string */
+        snprintf( str, sizeof( str ), fmt, arg_line_number );
+
+        /* handle the various output modes */
+        switch ( output_mode )
+        {
+        case ASCII:
+            RB_ASCII_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        case HTML:
+            RB_HTML_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        case LATEX:
+            RB_LaTeX_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        case RTF:
+            RB_RTF_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        case TROFF:
+            RB_TROFF_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        case XMLDOCBOOK:
+            RB_XMLDB_Generate_Item_Line_Number( dest_doc, str );
+            break;
+
+        default:
+            break;
+        }
+    }
+}
+
+/******/
+
 /****f* Generator/Generate_Item
  * SYNOPSIS
  */
 static void Generate_Item(
     FILE *f,
-    struct RB_Document *document,
     struct RB_header *header,
     struct RB_Item *cur_item,
     char *docname )
  /*
   * FUNCTION
   *   Generate the documentation for a single item.
+  *
+  * NOTE
+  *   This function is way too long...
+  *
   * SOURCE
   */
 {
     static int          dot_nr = 1;
     int                 line_nr;
     char               *dot_type = NULL;
-    FILE               *tool = NULL;    // Pipe handler to the tool we use
+    FILE               *tool = NULL;    /* Pipe handler to the tool we use */
     enum ItemType       item_type = cur_item->type;
     char               *name = configuration.items.names[item_type];
 
@@ -1814,19 +1893,19 @@ static void Generate_Item(
         struct RB_Item_Line *item_line = cur_item->lines[line_nr];
         char               *line = item_line->line;
 
-        // Plain item lines
+        /* Plain item lines */
         if ( !Works_Like_SourceItem( item_type ) &&
              ( item_line->kind == ITEM_LINE_PLAIN ) )
         {
             Format_Line( f, item_line->format );
             Generate_Item_Line( f, line, item_type, docname, header );
         }
-        // Last line
+        /* Last line */
         else if ( item_line->kind == ITEM_LINE_END )
         {
             Format_Line( f, item_line->format );
         }
-        // Normal Pipes
+        /* Normal Pipes */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_PIPE ) )
         {
@@ -1836,22 +1915,22 @@ static void Generate_Item(
                 Pipe_Line( f, line );
             }
         }
-        // Tool start
+        /* Tool start */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_TOOL_START ) )
         {
             Format_Line( f, item_line->format );
 
-            // Change to docdir
-            RB_Change_To_Docdir( document );
+            /* Change to docdir */
+            RB_Change_To_Docdir( docname );
 
-            // Open pipe to tool
-            tool = Open_Pipe( line );
+            /* Open pipe to tool */
+            tool = RB_Open_Pipe( line );
 
-            // Get back to working dir
+            /* Get back to working dir */
             RB_Change_Back_To_CWD(  );
         }
-        // Tool (or DOT) body
+        /* Tool (or DOT) body */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_TOOL_BODY ) )
         {
@@ -1860,119 +1939,124 @@ static void Generate_Item(
                 fprintf( tool, "%s\n", line );
             }
         }
-        // Tool end
+        /* Tool end */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_TOOL_END ) )
         {
-            // Close pipe
-            Close_Pipe( tool );
+            /* Close pipe */
+            RB_Close_Pipe( tool );
             tool = NULL;
         }
-        // DOT start
+        /* DOT start */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_DOT_START ) )
         {
             Format_Line( f, item_line->format );
 
-            // Get DOT file type
+            /* Get DOT file type */
             dot_type = RB_Get_DOT_Type(  );
 
             if ( dot_type )
             {
                 char                pipe_str[TEMP_BUF_SIZE];
 
-                // Change to docdir
-                RB_Change_To_Docdir( document );
-#if defined(RB_MSVC)
-                sprintf( pipe_str,
-                         "%s -T%s -o%s%d.%s", dot_name,
-                         dot_type, DOT_GRAPH_NAME, dot_nr, dot_type );
-#else
+                /* Change to docdir */
+                RB_Change_To_Docdir( docname );
                 snprintf( pipe_str, sizeof( pipe_str ),
                           "%s -T%s -o%s%d.%s", dot_name,
                           dot_type, DOT_GRAPH_NAME, dot_nr, dot_type );
-#endif
-                tool = Open_Pipe( pipe_str );
+                tool = RB_Open_Pipe( pipe_str );
             }
         }
-        // DOT end
+        /* DOT end */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_DOT_END ) )
         {
             if ( tool )
             {
-                // Close pipe
-                Close_Pipe( tool );
+                /* Close pipe */
+                RB_Close_Pipe( tool );
                 tool = NULL;
 
-                // Generate link to image
+                /* Generate link to image */
                 RB_Generate_DOT_Image_Link( f, dot_nr, dot_type );
 
-                // Get back to working dir
+                /* Get back to working dir */
                 RB_Change_Back_To_CWD(  );
 
-                // Increment dot file number
+                /* Increment dot file number */
                 dot_nr++;
             }
         }
-        // DOT file include
+        /* DOT file include */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_DOT_FILE ) )
         {
             Format_Line( f, item_line->format );
 
-            // Get DOT file type
+            /* Get DOT file type */
             dot_type = RB_Get_DOT_Type(  );
 
             if ( dot_type )
             {
+                FILE               *in = NULL, *dot_pipe = NULL;
                 char                str[TEMP_BUF_SIZE];
 
-                // Change to docdir
-                RB_Change_To_Docdir( document );
+                /* Open the dot source */
+                snprintf( str, sizeof( str ), "%s%s",
+                          header->owner->filename->path->name, line );
+                in = RB_Open_File( str, "r" );
 
-#if defined(RB_MSVC)
-                sprintf( str,
-                         "%s -T%s \"%s/%s\" -o%s%d.%s", dot_name,
-                         dot_type, RB_Get_Saved_CWD(  ), line,
-                         DOT_GRAPH_NAME, dot_nr, dot_type );
-#else
+                /* Change to docdir */
+                RB_Change_To_Docdir( docname );
+
+                /* Open a pipe to DOT */
                 snprintf( str, sizeof( str ),
-                          "%s -T%s \"%s/%s\" -o%s%d.%s", dot_name,
-                          dot_type, RB_Get_Saved_CWD(  ), line,
-                          DOT_GRAPH_NAME, dot_nr, dot_type );
-#endif
-                system( str );
+                          "%s -T%s -o%s%d.%s", dot_name,
+                          dot_type, DOT_GRAPH_NAME, dot_nr, dot_type );
+                dot_pipe = RB_Open_Pipe( str );
 
-                // Generate link to image
+                /* Pipe data to DOT */
+                while ( fgets( str, sizeof( str ), in ) != NULL )
+                    fputs( str, dot_pipe );
+
+                /* Close file handlers */
+                RB_Close_File( in );
+                RB_Close_Pipe( dot_pipe );
+
+                /* Generate link to image */
                 RB_Generate_DOT_Image_Link( f, dot_nr, dot_type );
 
-                // Get back to working dir
+                /* Get back to working dir */
                 RB_Change_Back_To_CWD(  );
 
-                // Increment dot file number
+                /* Increment dot file number */
                 dot_nr++;
             }
         }
-        // Exec item
+        /* Exec item */
         else if ( !Works_Like_SourceItem( item_type ) &&
                   ( item_line->kind == ITEM_LINE_EXEC ) )
         {
             Format_Line( f, item_line->format );
 
-            // Change to docdir
-            RB_Change_To_Docdir( document );
+            /* Change to docdir */
+            RB_Change_To_Docdir( docname );
 
-            // Execute line
+            /* Execute line */
             system( line );
 
-            // Get back to working dir
+            /* Get back to working dir */
             RB_Change_Back_To_CWD(  );
         }
-        // Source lines
+        /* Source lines */
         else if ( Works_Like_SourceItem( item_type ) )
         {
             Format_Line( f, item_line->format );
+            /* Generate line numbers for SOURCE like items */
+            RB_Generate_Item_Line_Number( f, item_line->line_number,
+                                          cur_item->max_line_number );
+            /* Generate item line */
             Generate_Item_Line( f, line, item_type, docname, header );
         }
         else
@@ -1995,7 +2079,6 @@ static void Generate_Item(
  */
 static void Generate_Header(
     FILE *f,
-    struct RB_Document *document,
     struct RB_header *header,
     char *docname )
 /*
@@ -2003,7 +2086,7 @@ static void Generate_Header(
  *   * f -- destination file
  *   * header -- header to be searched.
  *   * srcname -- name of the source file the header was found in.
- *   * document -- name of the documentation file.
+ *   * docname -- name of the documentation file.
  * BUGS
  *   This skips the first item body if the first item name was
  *   not correctly spelled.
@@ -2028,7 +2111,7 @@ static void Generate_Header(
         }
         else
         {
-            Generate_Item( f, document, header, cur_item, docname );
+            Generate_Item( f, header, cur_item, docname );
         }
     }
 }
@@ -2348,48 +2431,6 @@ static void Generate_Begin_Paragraph(
 }
 
 
-#if 0
-/*x**f* Generator/RB_Generate_Empty_Item
- * FUNCTION
- *   Generate documentation for an item with an empty body.
- * INPUTS
- *   dest_doc -- destination file.
- ****
- * TODO Documentation
- */
-
-static void RB_Generate_Empty_Item(
-    FILE *dest_doc )
-{
-    switch ( output_mode )
-    {
-    case TEST:
-        break;
-    case XMLDOCBOOK:
-        break;
-    case HTML:
-        RB_HTML_Generate_Empty_Item( dest_doc );
-        break;
-    case LATEX:
-        RB_LaTeX_Generate_Empty_Item( dest_doc );
-        break;
-    case RTF:
-        RB_RTF_Generate_Empty_Item( dest_doc );
-        break;
-    case TROFF:
-        break;
-    case ASCII:
-        RB_ASCII_Generate_Empty_Item( dest_doc );
-        break;
-    case UNKNOWN:
-    default:
-        assert( 0 );
-    }
-}
-#endif
-
-
-
 static void Generate_Item_Line(
     FILE *dest_doc,
     char *line,
@@ -2463,7 +2504,7 @@ static void Generate_Item_Line(
             {
                 if ( utf8_isspace( c ) )
                 {
-                    /* We found the end of the string, so we go 
+                    /* We found the end of the string, so we go
                      * back to the space skipping state
                      */
                     state = SKIP_SPACE;
@@ -2608,7 +2649,7 @@ static void Generate_Item_Line(
         break;
 
     case HTML:
-        // Check for source line comment endings
+        /* Check for source line comment endings */
         RB_HTML_Generate_Line_Comment_End( dest_doc );
         break;
 
@@ -2806,7 +2847,6 @@ static void Generate_Index(
         RB_HTML_Generate_Index( document );
         break;
     case LATEX:
-        /* RB_LaTeX_Generate_Index(dest, source); */
         /* Latex has a index by default */
         break;
     case UNKNOWN:
