@@ -27,7 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * TODO
  *   Documentation.
  *****
- * $Id: globals.c,v 1.23 2008/06/17 11:49:27 gumpu Exp $
+ * $Id: globals.c,v 1.24 2009/03/23 21:15:04 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -168,6 +168,9 @@ int                 header_breaks = DEFAULT_HEADER_BREAKS;
 char               *dot_name = DEFAULT_DOT_NAME;
 
 
+/* Name of item to add source comments */
+char               *use_source_comments = "";
+
 /****i* Globals/RB_Close_The_Shop [3.0b]
  * NAME
  *   RB_Close_The_Shop -- free resources.
