@@ -1,3 +1,25 @@
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
 /****h* ROBODoc/Headers
  * FUNCTION
  *   This module contains a set of variables that define how headers
@@ -25,7 +47,7 @@
 #define NO_MARKER        100002
 
 static int          locked_header_marker = NO_MARKER_LOCKED;
-static int          locked_end_marker = NO_MARKER_LOCKED;
+static int          locked_end_marker    = NO_MARKER_LOCKED;
 static int          locked_remark_marker = NO_MARKER_LOCKED;
 
 /****v* Headers/header_markers
@@ -34,9 +56,8 @@ static int          locked_remark_marker = NO_MARKER_LOCKED;
  * FUNCTION
  *   These specify what robodoc recognizes as the beginning
  * NOTE
- *   The numbers at the beginning of the lines make it easier
- *   to keep them in sync with the src_constants
- *   of a header.
+ *   The numbers at the beginning of the lines make it easier to keep
+ *   them in sync with the src_constants of a header.
  * SOURCE
  */
 
@@ -91,7 +112,7 @@ char               *robo_header = "/*ROBODOC*"; /* TODO Remove */
 /****/
 
 
-/****v* Headers/remark_markers [3.0h]
+/****v* Headers/remark_markers
  * NAME
  *   remark_markers
  * FUNCTION
@@ -305,35 +326,65 @@ int RB_Is_Begin_Marker(
         assert( 0 );
     }
 
-
-    if ( found && cur_char )
+    if ( found && ( *cur_char == '\0' ) )
     {
+        /* start of an header but nothing else */
+        found = FALSE;
+    }
+    else if ( !found ) 
+    {
+        /* Not found */
+    }
+    else 
+    {
+        /* We found the start of a header, now check the remaining
+         * part. */
+
+        /*             ****f*
+         * We are now here ^
+         */
         *type = cur_char;
         ++cur_char;
-        if ( *cur_char == '*' )
+        if ( *cur_char == '\0' ) 
         {
-            ++cur_char;
-            found = utf8_isspace( *cur_char );
-        }
-        else if ( *( cur_char + 1 ) == '*' )
-        {
-            ++cur_char;
-            ++cur_char;
-            found = utf8_isspace( *cur_char );
+            found = FALSE;
         }
         else
         {
-            found = FALSE;
-        }
-        if ( found )
-        {
-            found = FALSE;
-            /* It should contain some non * characters. */
-            for ( ; *cur_char; ++cur_char )
+            /* |           ****f*
+             * We are now here  ^
+             */
+            if ( *cur_char == '*' )
             {
-                if ( utf8_isalnum( *cur_char ) )
+                ++cur_char;
+                found = utf8_isspace( *cur_char );
+            }
+            else if ( *cur_char && ( *( cur_char + 1 ) == '*' ) )
+            {
+                /* The case ****if*
+                 */
+                ++cur_char;
+                ++cur_char;
+                /* |        ****if*
+                 * We are now here ^
+                 */
+                found = ( *cur_char ) && utf8_isspace( *cur_char );
+            }
+            else
+            {
+                found = FALSE;
+            }
+
+            if ( found )
+            {
+                found = FALSE;
+                /* It should contain some non * characters. */
+                for ( ; *cur_char; ++cur_char )
                 {
-                    found = TRUE;
+                    if ( utf8_isalnum( *cur_char ) )
+                    {
+                        found = TRUE;
+                    }
                 }
             }
         }
@@ -540,8 +591,7 @@ int RB_Is_End_Marker(
     /* Locking on end markers does not work at the moment,
      * because there can be more than one end marker for
      * a given language. TODO 
-     */
-#if 0
+
     if ( found &&
          ( course_of_action.do_LOCKHEADER ) &&
          ( locked_end_marker == NO_MARKER_LOCKED ) )
@@ -551,7 +601,8 @@ int RB_Is_End_Marker(
         RB_Say( "end marker locked on %s\n", SAY_INFO,
                 end_markers[locked_end_marker] );
     }
-#endif
+    */
+
     return found;
 }
 
@@ -681,8 +732,6 @@ char               *RB_Skip_Remark_Marker(
 /**************/
 
 
-
-
 /* TODO Documentation */
 void RB_Header_Lock_Reset(
     void )
