@@ -1,10 +1,35 @@
-// vi: ff=unix spell 
+/* vi: ff=unix spell
+ *
+ */
+
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
 /****h* ROBODoc/Utilities
  * FUNCTION
  *   Set of general purpose utility functions that are used
  *   in more than one module.
  *****
- * $Id: util.c,v 1.56 2007/05/09 07:41:05 thuffir Exp $
+ * $Id: util.c,v 1.63 2008/06/17 11:49:28 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -18,12 +43,7 @@
 #include <time.h>
 #include <assert.h>
 #include <sys/stat.h>
-
-#if defined (RB_MSVC)
-/* empty */
-#else
 #include <unistd.h>
-#endif
 
 #include "robodoc.h"
 #include "globals.h"
@@ -64,7 +84,6 @@ char               *ExpandTab(
  *   This function is not reentrant.
  * SOURCE
  */
-
 {
     char               *cur_char = line;
     int                 n = 0;
@@ -86,7 +105,7 @@ char               *ExpandTab(
         {
             int                 i;
 
-            // Seek to actual tab stop position in tabstop table
+            /* Seek to actual tab stop position in tabstop table */
             while ( ( tab_stops[actual_tab] <= n )
                     && ( actual_tab < ( MAX_TABS - 1 ) ) )
             {
@@ -95,7 +114,7 @@ char               *ExpandTab(
 
             jump = tab_stops[actual_tab] - n;
 
-            // If jump gets somehow negative fix it...
+            /* If jump gets somehow negative fix it... */
             if ( jump < 0 )
             {
                 jump = 1;
@@ -144,7 +163,6 @@ struct RB_header   *RB_Alloc_Header(
  *   RB_Free_Header()
  * SOURCE
  */
-
 {
     struct RB_header   *new_header;
 
@@ -180,7 +198,6 @@ void RB_Free_Header(
  *   RB_Alloc_Header(), RB_Close_The_Shop()
  * SOURCE
  */
-
 {
     if ( header )
     {
@@ -206,7 +223,7 @@ void RB_Free_Header(
 
             for ( i = 0; i < header->no_lines; ++i )
             {
-                free( header->lines[i] );
+                free( header->lines[i].line );
             }
             free( header->lines );
         }
@@ -294,7 +311,6 @@ void RB_Say(
  *   22. Nov. 2005   - Multiple mode support (Michel Albert)
  * SOURCE
  */
-
 {
     va_list             ap;
 
@@ -314,9 +330,9 @@ void RB_Say(
  * NAME
  *   RB_SetCurrentFile
  * FUNCTION
- *   Set... (TODO Documentation) 
+ *   Set... (TODO Documentation)
  * INPUTS
- *   * filename -- 
+ *   * filename --
  * SOURCE
  */
 
@@ -352,31 +368,6 @@ char               *RB_GetCurrentFile(
 /*******/
 
 
-
-/****f* Utilities/Path2Win32Path
- * SUMMARY
- *   Convert a path to a path in MS windows format.
- ****
- */
-
-char               *Path_2_Win32Path(
-    char *path )
-{
-    char               *cur_char = path;
-
-    for ( ; *cur_char; ++cur_char )
-    {
-        if ( *cur_char )
-        {
-            if ( *cur_char == '/' )
-            {
-                *cur_char = '\\';
-            }
-        }
-    }
-    return path;
-}
-
 /****f* Utilities/RB_Panic [2.01]
  * NAME
  *   RB_Panic -- free resources and shut down
@@ -397,7 +388,6 @@ void RB_Panic(
  *   Koessi
  * SOURCE
  */
-
 {
     va_list             ap;
     char               *name;
@@ -406,13 +396,6 @@ void RB_Panic(
 
     name = RB_GetCurrentFile(  );
 
-#if defined(RB_BCC)
-    if ( course_of_action.do_ms_errors )
-    {
-        Path_2_Win32Path( name );
-    }
-#endif
-
     if ( name )
     {
         char               *buffer_copy = RB_StrDup( myLine );
@@ -495,13 +478,6 @@ void RB_Warning(
 
     name = RB_GetCurrentFile(  );
 
-#if defined (RB_BCC)
-    if ( course_of_action.do_ms_errors )
-    {
-        Path_2_Win32Path( name );
-    }
-#endif
-
     if ( name )
     {
         fprintf( stderr, "%s:\n%s(%d) : Warning R%d:\n", whoami, name,
@@ -575,11 +551,11 @@ void RB_TimeStamp(
  *   char * RB_Skip_Whitespace(char *buf)
  * FUNCTION
  *   Skip space and tab chars from the start *buf. This is needed when
- *   searching for indented headers and items. 
+ *   searching for indented headers and items.
  * NOTES
  *   We should extract some info about indentation level and save it to
  *   global variable in order to write out source items (that originate from
- *   indented headers) neatly. 
+ *   indented headers) neatly.
  * SEE ALSO
  *   RB_Find_Marker, RB_Find_End_Marker, RB_Find_Item, RB_Generate_Item_Body
  * SOURCE
@@ -784,7 +760,7 @@ int RB_Match(
             int                 l_t = strlen( target );
 
             /* The minimum of the length of the wildcard_expression
-             * and target expression 
+             * and target expression
              */
             int                 l = ( l_w <= l_t ) ? l_w : l_t;
             int                 i;
@@ -851,7 +827,7 @@ static void RB_Swap(
  * FUNCTION
  *   Sort an array of pointers according to the lexical order
  *   of the elements the pointers point to.
- *   This is based on the quicksort routine in 
+ *   This is based on the quicksort routine in
  *   "The C programming language" by B Kerninghan en D Ritchie.
  * INPUTS
  *   * array -- the array of pointers.
@@ -868,8 +844,8 @@ static void RB_Swap(
  *   the use
  *    #define TEST_SIZE 10
  *
- *    char* test[ TEST_SIZE ] = { "ape", "zebra", 
- *       "duck", "goofbal", "dodo", "rabit", 
+ *    char* test[ TEST_SIZE ] = { "ape", "zebra",
+ *       "duck", "goofbal", "dodo", "rabit",
  *       "crow", "cow", "pig", "goat" };
  *
  *    int string_compare( void* p1, void* p2 )
@@ -879,7 +855,7 @@ static void RB_Swap(
  *       return strcmp( cp1, cp2 );
  *    }
  *
- *    RB_QuickSort( test, 0, TEST_SIZE - 1, string_compare ); 
+ *    RB_QuickSort( test, 0, TEST_SIZE - 1, string_compare );
  *
  * SOURCE
  */
@@ -1004,21 +980,21 @@ void RB_FreeLineBuffer(
  * SOURCE
  */
 
-static int CR_LF_Conversion( char* line )
+static int CR_LF_Conversion(
+    char *line )
 {
-    int n = strlen( line );
+    int                 n = strlen( line );
 
-    if ( ( n > 1 ) && 
-          ( ( ( line[ n - 2 ] == '\r' ) && ( line [ n - 1 ] == '\n' ) ) 
-            ||
-            ( ( line[ n - 2 ] == '\n' ) && ( line [ n - 1 ] == '\r' ) ) 
-          )
-       )
+    if ( ( n > 1 ) &&
+         ( ( ( line[n - 2] == '\r' ) && ( line[n - 1] == '\n' ) )
+           || ( ( line[n - 2] == '\n' ) && ( line[n - 1] == '\r' ) ) ) )
     {
-        line[ n - 2 ] = '\n';
-        line[ n - 1 ] = '\0';
+        line[n - 2] = '\n';
+        line[n - 1] = '\0';
         return 1;
-    } else {
+    }
+    else
+    {
         /* No problem */
         return 0;
     }
@@ -1029,7 +1005,7 @@ static int CR_LF_Conversion( char* line )
 
 /****f* Utilities/RB_ReadWholeLine
  * FUNCTION
- *   Read a line from the file using the provided buffer.
+ *   Read a single line from the file using the provided buffer.
  * INPUTS
  *   * file -- file to read from
  *   * buf -- buffer of length MAX_LINE_LEN to read chunks of the line to
@@ -1049,7 +1025,7 @@ char               *RB_ReadWholeLine(
 {
     int                 foundNL = 0;
     char               *line = NULL;
-    int                 curLineLen = 1;
+    int                 curLineLen = 0;
     int                 chunkLen = 0;
 
     clearerr( file );
@@ -1065,14 +1041,14 @@ char               *RB_ReadWholeLine(
         }
         chunkLen = strlen( buf );
         curLineLen += chunkLen;
-        /* make room for the chunk in our buffer */
-        if ( ( line = realloc( line, sizeof( char ) * curLineLen ) ) == NULL )
+        /* make room for the chunk in our buffer  ( +1 for the '\0') */
+        if ( ( line = realloc( line, sizeof( char ) * ( curLineLen + 1 ) ) ) == NULL )
         {
             /* we run out of memory */
             RB_Panic( "Out of memory! RB_ReadWholeLine()" );
         }
         /* append the chunk to our buffer */
-        strcpy( ( line + curLineLen - chunkLen - 1 ), buf );
+        strcpy( ( line + curLineLen - chunkLen ), buf );
 
         if ( RB_ContainsNL( buf ) )
         {
@@ -1085,13 +1061,15 @@ char               *RB_ReadWholeLine(
     {
         /* last line has no NL - add one */
         ++curLineLen;
-        if ( ( line = realloc( line, sizeof( char ) * curLineLen ) ) == NULL )
+
+        /* + 1 for the '\0' */
+        if ( ( line = realloc( line, sizeof( char ) * ( curLineLen + 1 ) ) ) == NULL )
         {
             /* we run out of memory */
             RB_Panic( "Out of memory! RB_ReadWholeLine()" );
         }
-        line[curLineLen - 2] = '\n';
-        line[curLineLen - 1] = '\0';
+        line[curLineLen - 1] = '\n';
+        line[curLineLen ] = '\0';
     }
 
     /* This fixes any cr/lf problems. */
@@ -1099,6 +1077,7 @@ char               *RB_ReadWholeLine(
 
     *arg_readChars = curLineLen;
     *buf = '\0';
+
     return line;
 }
 
@@ -1197,85 +1176,6 @@ void               *RB_malloc(
 /*******/
 
 
-/****v* Utilities/RB_crc32_table
- * FUNCTION
- *   CRC32 lookup table for crc32 functions
- * SOURCE
- */
-static unsigned long *RB_crc32_table;
-
-/*******/
-
-
-/****f* Utilities/Make_crc32_table
- * FUNCTION
- *   Builds up crc32 lookup table for crc32 function.
- *   Call this before calling to RB_crc32()
- * SOURCE
- */
-#define CRC32_GENERATOR_POLYNOMIAL      0xEDB88320
-
-void Make_crc32_table(
-    void )
-{
-    unsigned int        i, n, carry;
-    unsigned long       tmp;
-
-    // Allocate space for CRC table
-    RB_crc32_table = RB_malloc( 256 * sizeof( unsigned long ) );
-
-    for ( i = 0; i < 256; i++ )
-    {
-        tmp = i;
-        for ( n = 0; n < 8; n++ )
-        {
-            carry = tmp & 1;
-            tmp >>= 1;
-            if ( carry )
-            {
-                tmp ^= CRC32_GENERATOR_POLYNOMIAL;
-            }
-        }
-        RB_crc32_table[i] = tmp;
-    }
-}
-
-/*******/
-
-
-/****f* Utilities/RB_crc32
- * FUNCTION
- *   calcualtes crc32 value for a given buffer
- * INPUTS
- *   - buf:   input buffer to calculate
- *   - len:   length of buffer
- *   - crc32: CRC32 init value
- * RETURN VALUE
- *   CRC32 value
- * SOURCE
- */
-unsigned long RB_crc32(
-    unsigned char *buf,
-    unsigned int len,
-    unsigned long crc32 )
-{
-    unsigned char       tmp;
-
-    crc32 ^= 0xffffffffL;
-
-    while ( len-- )
-    {
-        tmp = *( buf++ ) ^ crc32;
-        crc32 >>= 8;
-        crc32 ^= RB_crc32_table[tmp];
-    }
-
-    return crc32 ^= 0xffffffffL;
-}
-
-/*******/
-
-
 /****f* Utilities/cwd
  * FUNCTION
  *   Holds current working directory
@@ -1285,19 +1185,6 @@ static char        *cwd = NULL;
 
 /*******/
 
-/****f* Utilities/RB_Get_Saved_CWD
- * FUNCTION
- *   Changes back to saved working directory and frees cwd string
- * SOURCE
- */
-char               *RB_Get_Saved_CWD(
-    void )
-{
-    return cwd;
-}
-
-/*******/
-
 /****f* Utilities/RB_Change_Back_To_CWD
  * FUNCTION
  *   Changes back to saved working directory and frees cwd string
@@ -1322,51 +1209,44 @@ void RB_Change_Back_To_CWD(
  * SOURCE
  */
 void RB_Change_To_Docdir(
-    struct RB_Document *document )
+    char *docname )
 {
-    char                tmp[TEMP_BUF_SIZE];
+    char                tmp[TEMP_BUF_SIZE], *namestart;
+    int                 len;
 
-    // Just for sure
+    /* Check if we have a valid directory name in the docname */
+    namestart = strrchr( docname, '/' );
+    if ( namestart == NULL )
+        RB_Panic( "Unable to get the directory name of '%s'", docname );
+
+    /* Just for sure */
     RB_Change_Back_To_CWD(  );
 
-    // Save CWD
+    /* Save CWD */
     getcwd( tmp, sizeof( tmp ) );
     cwd = RB_StrDup( tmp );
 
-    // Get the name of doc directory
-    // I think this is not the right way to do it. Anyone help plz?
-    if ( course_of_action.do_multidoc )
-    {
-        chdir( document->docroot->name );
-    }
-    else if ( course_of_action.do_singledoc )
-    {
-        int                 len = strrchr( document->singledoc_name, '/' )
-            - document->singledoc_name;
-        strncpy( tmp, document->singledoc_name, len );
-        tmp[len] = 0;
-        chdir( tmp );
-    }
+    /* Get the name of doc directory and change into it */
+    len = namestart - docname;
+    strncpy( tmp, docname, len );
+    tmp[len] = 0;
+    chdir( tmp );
 }
 
 /*******/
 
 
-/****f* Utilities/Open_Pipe
+/****f* Utilities/RB_Open_Pipe
  * FUNCTION
  *   Opens a pipe and returns its handler
  * SOURCE
  */
-FILE               *Open_Pipe(
+FILE               *RB_Open_Pipe(
     char *pipe_name )
 {
     FILE               *a_pipe;
 
-#if defined(RB_MSVC)
-    a_pipe = _popen( pipe_name, "w" );
-#else
     a_pipe = popen( pipe_name, "w" );
-#endif
 
     if ( a_pipe == NULL )
     {
@@ -1378,21 +1258,58 @@ FILE               *Open_Pipe(
 /*******/
 
 
-/****f* Utilities/Close_Pipe
+/****f* Utilities/RB_Close_Pipe
  * FUNCTION
  *   Closes a given pipe
  * SOURCE
  */
-void Close_Pipe(
+void RB_Close_Pipe(
     FILE *arg_pipe )
 {
     if ( arg_pipe != NULL )
     {
-#if defined(RB_MSVC)
-        _pclose( arg_pipe );
-#else
         pclose( arg_pipe );
-#endif
+    }
+}
+
+/*******/
+
+
+/****f* Utilities/RB_Open_File
+ * FUNCTION
+ *   Opens a file and returns its handler
+ * SOURCE
+ */
+FILE               *RB_Open_File(
+    char *file_name,
+    char *mode )
+{
+    FILE               *a_file;
+
+    a_file = fopen( file_name, mode );
+
+    if ( a_file == NULL )
+    {
+        RB_Panic( "Unable to open file '%s' with mode '%s'", file_name,
+                  mode );
+    }
+    return a_file;
+}
+
+/*******/
+
+
+/****f* Utilities/RB_Close_File
+ * FUNCTION
+ *   Closes a given file
+ * SOURCE
+ */
+void RB_Close_File(
+    FILE *arg_file )
+{
+    if ( arg_file != NULL )
+    {
+        fclose( arg_file );
     }
 }
 
@@ -1414,8 +1331,7 @@ int utf8_isalpha(
 {
     /* We assume here that all non ascii characters
      * are alphabetic characters. */
-    return ( ( arg_c >= 128 ) ||
-             ( arg_c < 128 && isalpha( arg_c ) ) );
+    return ( ( arg_c >= 128 ) || ( arg_c < 128 && isalpha( arg_c ) ) );
 }
 
 int utf8_iscntrl(
@@ -1471,3 +1387,32 @@ int utf8_isxdigit(
 {
     return ( ( arg_c < 128 ) && isxdigit( arg_c ) );
 }
+
+
+/****f* Utilities/snprintf
+ * FUNCTION
+ *   Mimic the library function snprintf using sprintf if it is absent.
+ * SOURCE
+ */
+#ifndef HAVE_SNPRINTF
+int snprintf(
+    char *str,
+    size_t size,
+    const char *format,
+    ... )
+{
+    va_list             ap;
+    int                 retval;
+
+    /* This variable remains unused... */
+    USE( size );
+
+    /* Mimic snprintf */
+    va_start( ap, format );
+    retval = sprintf( str, format, ap );
+    va_end( ap );
+
+    return ( retval );
+}
+#endif
+/*******/
